from multiprocessing import Process, Queue, Lock
import numpy as np
import time
import sys
from scipy.stats import bernoulli
from prob_cond_true_freq import prob_cond_true_freq
from comp_spectrum import comp_spectrum

#function that is parallelized
def process_probCond(qinput,qoutput,lock,pileup_prefix,min_count,min_minor,ratio,n,ancestral):
    pileup = open(pileup_prefix+'.in', 'r')
    for item in iter(qinput.get,'STOP'):
        l = []
        lock.acquire()
        pileup.seek(item[0])
        for i in range(item[1]):
            l.append(pileup.readline())
        #...
        lock.release()
        p_list = []
        for l_item in l:
	    buf=l_item.split()
	    nder=int(buf[2])
	    ntot=int(buf[3])
	    unfolded=(1-int(buf[4]))*(ancestral=='provided')
            if ntot >= min_count and nder >= min_minor and (ntot-nder) >= min_minor:
                if bernoulli.rvs(1./ratio) == 1:
                    p = prob_cond_true_freq(n,nder,ntot,unfolded);
		    if np.sum(p) > 0:
                    	p_list.append(p)
        #...
        if len(p_list) != 0: #in case that all the lines parsed are not valid 
            qoutput.put(p_list)           
    pileup.close()
#...

def prob_cond(min_count,min_minor,region,theta,nProcess,ratio,n,prefix,pileup_prefix,ancestral):
    lock = Lock()
    task_queue = Queue()
    done_queue = Queue()
    block = 10000
    pileup = open(pileup_prefix+'.in', 'rb')

    if region:
        chro = region[0]
        start = region[1]
        end = region[2]
        offset_default = pileup.tell()
        pileup_line = pileup.readline()
        a = pileup_line.split()[0]
        while(a != chro):
            offset_default = pileup.tell()
            pileup_line = pileup.readline()
            try:
                a = pileup_line.split()[0]
            except IndexError:
                #if the pileup_line can't be splited, that's the end of the file
                print ('ERROR : chro %s not found' % (chro))
                sys.exit()
            #...
        #...
        if start:
            a = int(pileup_line.split()[1])
            b = pileup_line.split()[0]
            if a > end:
                print ('ERROR : interval\'s positions not found.')
                sys.exit()
            #...
            while a < start and b == chro:
                offset_default = pileup.tell()
                pileup_line = pileup.readline()
                try:
                    a = int(pileup_line.split()[1])
                    b = pileup_line.split()[0]
                except IndexError:
                    #if the pileup_line can't be splited, that's the end of the file
                    print ('ERROR : interval\'s positions not found.')
                #...
            #...
            if b != chro:
                print ('ERROR : interval\'s positions not found.')
                sys.exit()
        #...
        offset_table = [offset_default]
        nbLine = 0
        split_pileup = pileup_line.split()
        while split_pileup[0] == chro:
            if start:
                if int(split_pileup[1]) > end:
                    break
                #...
             #...
            nbLine += 1
            if nbLine % block == 0:
                offset_table.append(pileup.tell())
            #...
            pileup_line = pileup.readline()
            split_pileup = pileup_line.split()
            if len(split_pileup) == 0:
                break
            #... 
        #...
    #...
    else:
        offset_table = [0]
        nbLine = 0
        pileup_line = pileup.readline()
        while(pileup_line != ''): #if pileup_line == '', that's the end of the file
            nbLine += 1
            if nbLine % block == 0:
                offset_table.append(pileup.tell())
            #...
            pileup_line = pileup.readline()
        #...
    #...
    pileup.close()

    #for each offset except the last one
    for offset in offset_table[:-1]:
        task_queue.put([offset,block])
    #...
    
    #management of the last line_block 
    if nbLine % block != 0:
        task_queue.put([offset_table[-1],nbLine % block])
    #...
    
    del offset_table
    
    for i in range(nProcess):
        task_queue.put('STOP')
    #...
    
    for i in range(nProcess):
        p = Process(target=process_probCond,args=(task_queue,done_queue,lock,pileup_prefix,min_count,min_minor,ratio,n,ancestral)).start()  
    #...
    
    while task_queue.qsize() != 0:
        pass
    #...
    
    p_neutral = []
    for i in range(done_queue.qsize()):
        p_neutral += done_queue.get()
    #...
    
    p_neutral = np.array(p_neutral)
    
    p_neutral = comp_spectrum(p_neutral,n,theta,ancestral)
    np.savetxt(prefix + '.spectrum', np.array([p_neutral]),delimiter=' ', fmt='%.6e')
    return p_neutral
#...
